export default {
	data() {
		return {
			map: null,  // la carte leaflet associée au composant
			marker: null,  // le 
		};
	},
	computed : {
		markerLatitude() {
			return (this.marker)?this.marker.getLatLng().lat:"";
		},
		markerLongitude() {
			return (this.marker)?this.marker.getLatLng().lng:"";
		}
	},
	props: {  // propriétés externes pouvant être passées au composant depuis un composant parent
		title: String,   // titre associé à la carte
		mapid: { type: String, required: true }, // identifiant du div supportant la carte
		latitude: { type: Number, default: 0 }, // propriété permettant de fixer la latitude du marqueur
		longitude: { type: Number, default: 0 }, // propriété permettant de fixer la longitude du marqueur
		zoomlevel: { type: Number, default: 8 }, // propriété permettant de fixer le niveau de zoom de la carte
	},
	emits: ["marker-change"],  // événéments émis par le composant
	methods: {
		/**
		 * méthode invoquée lorsque l'utilisateur clique sur la carte
		 * le marqueur associé à la carte est positionné sur le point cliqué et la carte
		 * est centrée sur celui-ci.
		 * Un évenement 'marker-change' est émis par le composant. Les données associées à cet événemment
		 * sont regroupées dans un objet dont les propriétés sont :
		 * - mapid : l'identifiant de la carte
		 * - latitude : la latitude du point cliqué (arrondie à 3 décimales)
		 * - longitude : la longitude du point cliqué (arrondie à 3 décimales)
		 * @param {*} e : l'évenement émis par leaflet lorsque l'utilisateur clique sur la carte
		 *                (voir la doc leaflet : https://leafletjs.com/reference.html#mouseevent)
		 */
		onMapClick(e) {
			let lat = parseFloat(e.latlng.lat.toFixed(5));
			let lng = this.normalizeLongitude(parseFloat(e.latlng.lng.toFixed(5)));
			this.marker.setLatLng([lat, lng]);
			this.map.setView([lat, lng]);
			this.$emit("marker-change", {
				mapid: this.mapid,
				latitude: lat,
				longitude: lng,
			});
		},
		/**
		 * @param {number} lng 
		 * @returns longitude normalisée , valeurs toujours comprise entre -180 et 180
		 */
		normalizeLongitude(lon) {
            while (lon < -180) {
                lon += 360;
            }
            while (lon > 180) {
                lon -= 360;
            }
            return parseFloat(lon.toFixed(5));
        }
	},
	watch: {
		/**
		 * cette fonction est exécutée chaque fois que la propriété zoomLevel change.
		 * @param {Number} newZoomlevel : nouvelle valeur pour le niveau de zoom
		 */
		zoomlevel(newZoomlevel) {
			this.map.setZoomAround(this.marker.getLatLng(), newZoomlevel);
		},
		/**
		 * cette fonction est exécutée chaque fois que la propriété latitude change
		 * @param {Number} newLatitude  
		 */
		latitude(newLatitude) {
			let lng = this.marker.getLatLng().lng;
			this.map.setView([newLatitude, lng]);
			this.marker.setLatLng([newLatitude, lng]);
		},
		/**
		 * cette fonction est exécutée chaque fois que la propriété longitude change
		 * @param {Number} newLongitude 
		 */
		longitude(newLongitude) {
			let lat = this.marker.getLatLng().lat;
			this.map.setView([lat, newLongitude]);
			this.marker.setLatLng([lat, newLongitude]);
		},
	},
	mounted() {
		// méthode invoquée quand le composant Vue a été monté
		// création de la carte Leaflet
		this.map = L.map(this.mapid, {
			center: [this.latitude, this.longitude], // initialise la carte centrée sur this.latitude, this.longitude
			zoom: this.zoomlevel,  // fixe le niveau de zoom intial pour la carte
			zoomControl: false,    // supprime les contrôles de zoom avant et arrière sur la carte
			scrollWheelZoom: false,  // désactive le zoom avec la molette de la souris
			dragging: false,       //  empêche l'utilisateur de déplacer la carte

		});
        // fond de carte
		L.tileLayer("https://{s}.tile.openstreetmap.org/{z}/{x}/{y}.png", {
			attribution: 'Map data © <a href="https://openstreetmap.org">OpenStreetMap</a> contributors',
			maxZoom: 19,
			tileSize: 512,
			zoomOffset: -1,
		}).addTo(this.map);
        // affiche le marqueur associé à la carte
		this.marker = L.marker([this.latitude, this.longitude]).addTo(this.map);
        // définit la méthode onMapClick du composant comme gestionnaire (handler) des clics sur la carte
		this.map.on("click", this.onMapClick);
	},
	template: `
    <div class="map-div">
        <h2 v-if="title">{{ title }}</h2>
        <div class="map-canvas" :id="mapid">
        </div>
		<div class="marker-coords">
		    latitude : {{markerLatitude}} - longitude : {{markerLongitude}} - zoom level : {{zoomlevel}}
		</div>
    </div>
    `,
};
